function PainelController($scope, $interval, ApplicationContext, ScreenService) {
    var sf = $scope;
    sf.dadosModificados = [];

    // Verifica se o onSwipe está 'true', para abrir o swipe.
    $scope.changeOnSwipe = function(){
        let changeSwipe = $scope.widget.actions.filter(function(object){
            if(object.onSwipe != undefined) return object;    
        });
        return changeSwipe[0].onSwipe;
    };
    
    // Sempre que o dataSource mudar, ele atualiza a tela, mantendo o estado dos elementos.
    $scope.$watch(() => $scope.widget.dataSource.data, () => {
        let widget = $scope.widget;
        initVetorProduct(widget);
        if(sf.intervalAttPainel != undefined) $interval.cancel(sf.intervalAttPainel);
        atualizaPainel();
    }, true);

    // Inicia o array de Produtos.
     function initVetorProduct(widget){
        let data = widget.dataSource.data;
        $scope.productsObject = [];
        data.forEach(function(product){
            let QTAPRODUZIR = parseFloat(product.QTAPRODUZIR).toFixed(3);
            let time = true;
            let bar = false;
            // Verifica se não possui tempo atribuido.
            if(product.TEMPOTOTAL == '00:00') {
                time = false;
                bar = true;
            }

            // Verifica se o painel ja foi inicial anteriormente e continua a produção.
            let started = null; 
            let endHour = null;
            let running = false;
            if(product.HRPAINELINI != '00:00'){
                var tempoPrep = moment(product.TEMPOTOTAL, 'hh,mm');
                running = true;
                started = moment(product.HRPAINELINI, 'hh,mm');
                endHour = moment(started);

                endHour.add(tempoPrep.minute(), 'minute');
                endHour.add(tempoPrep.hour(), 'hour');
            }
            // QTAPRODUZIR = QTAPRODUZIR.replace('.', ',');

            var object = {
                isRunning: running, 
                isTime: time,
                initDelayed: false,
                delayed: false,
                progress: '100%',
                noBar: bar,
                hasImg: false,
                destroyDelayed: false,
                nrplanprod: product.NRPLANPROD,
                cdprodacabado: product.CDPRODACABADO,
                idcookchill: product.IDCOOKCHILL,
                idfinalizacao: product.IDFINALIZACAO,
                idtpprodchill: product.IDTPPRODCHILL,
                idestado: product.IDESTADO,
                nrseqprodacabado: product.NRSEQPRODACABADO,
                productName: product.NMPRODUTO, 
                produzir: QTAPRODUZIR, 
                unidade: product.SGUNIDADE,
                tempoTotal: moment(product.TEMPOTOTAL, 'hh,mm'),
                timeInit: moment(product.HORA_INICIO, 'hh,mm'), 
                timeLimit: moment(product.HORA_FIM, 'hh,mm'),
                hourStart: started,
                hourEnd: endHour,
                icon: true,
                durationProgress: null,
                countTimer: 0,
                intervalTime: null,
                labelIniciar: ScreenService.i18n('Iniciar'),
                labelFinalizar: ScreenService.i18n('Finalizar')
            }; 

            // Caso o horario inicial e o final sejam do dia anterior.
            if(product.HORA_INICIO_FLOAT < 0)
                object.timeInit.subtract(1, 'days');
            if(product.HORA_FIM_FLOAT < 0)
                object.timeLimit.subtract(1, 'days');
            
            if(object.isRunning && object.tempoTotal.format('HH:mm') != '00:00'){
                object.initDelayed = true;
                object.delayed = false;
                countInit(object);
                countdownClock(object);
            }

            $scope.productsObject.push(object);
        });
    }
    
    // Calcula o progresso da barra de tempo.
    function calcProgress(arrayElement){
        let timeActual = moment().valueOf();
        let percent = ((timeActual - arrayElement.hourStart.valueOf()) * 100) / (arrayElement.hourEnd.valueOf() - arrayElement.hourStart.valueOf());
        percent = percent <= 0 ? 0 : percent;
        percent = 100 - percent;
        return Math.round(percent) + '%';
    }
    
    // Verifica se o painel está atrasado.
    function delayed(products) {
        for (var i = 0; i < products.length; i++ ) {
            var arrayElement = products[i];
            var timeActual = moment().valueOf();
            var formatTotalTime = arrayElement.tempoTotal.format('HH:mm');
            
            if(timeActual >= arrayElement.timeInit.valueOf() && arrayElement.isRunning == false && 
            arrayElement.hourStart == null && formatTotalTime != '00:00'){
                arrayElement.delayed = true;
                arrayElement.isTime = false;
            }else if(timeActual >= arrayElement.timeLimit.valueOf() && arrayElement.hourStart == null && 
            formatTotalTime != '00:00'){
                arrayElement.delayed = true;
                arrayElement.destroyDelayed = true;
            }
        }
    }
    // Cria um intervalo para verificar se o painel está atrasado.
    $scope.checkDelayed = function(){
        let products = sf.productsObject;
        let stopDelayed;
        if(products){
            stopDelayed = $interval(delayed(products), 1000);
            products.forEach(function(){
               $interval.cancel(stopDelayed); 
            });
        }
    };
    
    // Inicia a barra de progressão do painel
    function countInit(arrayElement){
        let interval = 1000;
        let progressInterval = $interval(function() {
            if(arrayElement.hourStart != null)
                arrayElement.progress = calcProgress(arrayElement);

            if(arrayElement.progress == '0%') $interval.cancel(progressInterval);
        }, interval);
    }
    
    // Inicia o painel.
    $scope.start = function(arrayElement){
        var formatTotalTime = arrayElement.tempoTotal.format('HH:mm');
        if(arrayElement.delayed)
            arrayElement.delayed = false;
        
        let startAction = $scope.widget.actions.find(act=>act.startButton);
        
        let start = moment();
        let end = moment();
        end.add(arrayElement.tempoTotal.minute(), 'minute');
        end.add(arrayElement.tempoTotal.hour(), 'hour');
        
        arrayElement.hourStart = start;
        arrayElement.hourEnd = end;
        
        $scope.$eval(startAction.onClick, Object.assign({
            'arrayElement':arrayElement
        }, ApplicationContext));
        
        if(arrayElement && formatTotalTime != '00:00'){
            arrayElement.initDelayed = true;
            countdownClock(arrayElement);
            countInit(arrayElement);
        } 
    };   
    
    // Finaliza o painel.
    $scope.stop = function(arrayElement){
        let finalButton = $scope.widget.actions.find(act=>act.finalButton);
        sf.dadosModificados = sf.productsObject.filter(function(object) {
          return object.progress > '0';
        });
        
        $scope.$eval(finalButton.onClick, Object.assign({
            'arrayElement': arrayElement
        }, ApplicationContext));
    };
    
    // Atualiza o painel a cada 2 min
	function atualizaPainel(){
	    sf.intervalAttPainel = $interval(function() {
	        let attDataSource = $scope.widget.actions.find(act=>act.attDataSource);
            $scope.$eval(attDataSource.onClick, Object.assign({
                'productsObject': sf.productsObject
            }, ApplicationContext));
            
        }, 120000); //300000 -> 5min em millisec | 120000 -> 2min em millisec
    }

    // Time Countdown de acordo com o tempo de preparo de cada um. 
    function countdownClock(arrayElement){
        let interval = 1000;
        // Cria a duração do relogio de acordo com tempo do prato.
        attClockTime(arrayElement);
        var duration = moment.duration({
            'hour': arrayElement.tempoTotal.hours(),
            'minutes': arrayElement.tempoTotal.minutes(),
            'seconds': arrayElement.tempoTotal.seconds(),
            'milliseconds': arrayElement.tempoTotal.milliseconds(),
        });
        // Cria um intervalo de tempo para atualizar decrementar o relogio.
        let stopClock = $interval(function(){
            let timeActual = moment();
            
            duration = moment.duration(duration.asMilliseconds() - interval, 'milliseconds');
            arrayElement.tempoTotal.hours(duration.hours());
            arrayElement.tempoTotal.minutes(duration.minutes());
            arrayElement.tempoTotal.seconds(duration.seconds());
            // Mata o intervalo do relogio assim que atingir o tempo.
            if(timeActual >= arrayElement.hourEnd) {
                arrayElement.delayed = true;
                $interval.cancel(stopClock);
                delayCounter(arrayElement);
            }else if(!arrayElement.isRunning) $interval.cancel(stopClock);
        }, interval);
    }

    // Conta quanto tempo o prato está atrasado.
    function delayCounter(arrayElement){
        let interval = 1000;
        arrayElement.icon = false;
        let negativeTime = moment();
        if(negativeTime > arrayElement.hourEnd){
            negativeTime.subtract(arrayElement.hourEnd.hours(), 'hours');
            negativeTime.subtract(arrayElement.hourEnd.minutes(), 'minutes');
            negativeTime.subtract(arrayElement.hourEnd.seconds(), 'seconds');
            arrayElement.countTimer = negativeTime.seconds();
        }
        let intervalTime = $interval(function(){
            if(!arrayElement.isRunning)$interval.cancel(intervalTime);
            arrayElement.tempoTotal.hour(negativeTime.hours()).minute(negativeTime.minutes()).second(arrayElement.countTimer++);
        }, interval);
    }

    function attClockTime(arrayElement){
        // Se a hora iniciada for superior a hora atual o painel ira começa a contar errado.
        let different = moment();
        var painelHour = moment(arrayElement.hourStart, 'hh,mm');
        different.subtract(painelHour.hours(), 'hours');
        different.subtract(painelHour.minutes(), 'minutes');
        different.subtract(painelHour.seconds(), 'seconds');
        different.subtract(painelHour.milliseconds(), 'milliseconds');
        arrayElement.tempoTotal.subtract(different.hours(), 'hours');
        arrayElement.tempoTotal.subtract(different.minutes(), 'minutes');
        arrayElement.tempoTotal.subtract(different.seconds(), 'seconds');
        arrayElement.tempoTotal.subtract(different.milliseconds(), 'milliseconds');
    }
}